@@ -18,6 +18,7 @@ import subprocess
 
 from snakemake import snakemake
 from snakemake.shell import shell
+from snakemake.common import ON_WINDOWS
 
 
 def dpath(path):
@@ -26,8 +27,12 @@ def dpath(path):
     return os.path.realpath(join(os.path.dirname(__file__), path))
 
 
-def md5sum(filename):
-    data = open(filename, "rb").read()
+def md5sum(filename, ignore_newlines=False):
+    if ignore_newlines:
+        with open(filename, "r", encoding="utf-8", errors="surrogateescape") as f:
+            data = f.read().encode("utf8", errors="surrogateescape")
+    else:
+        data = open(filename, "rb").read()
     return hashlib.md5(data).hexdigest()
 
 
@@ -79,6 +84,7 @@ def run(
     set_pythonpath=True,
     cleanup=True,
     conda_frontend="mamba",
+    container_image="snakemake/snakemake:latest",
     **params
 ):
     """
@@ -156,13 +162,24 @@ def run(
                 continue
             targetfile = join(tmpdir, resultfile)
             expectedfile = join(results_dir, resultfile)
+
+            if ON_WINDOWS:
+                if os.path.exists(join(results_dir, resultfile + "_WIN")):
+                    continue  # Skip test if a Windows specific file exists
+                if resultfile.endswith("_WIN"):
+                    targetfile = join(tmpdir, resultfile[:-4])
+            elif resultfile.endswith("_WIN"):
+                # Skip win specific result files on Posix platforms
+                continue
+
             assert os.path.exists(targetfile), 'expected file "{}" not produced'.format(
                 resultfile
             )
             if check_md5:
-                # if md5sum(targetfile) != md5sum(expectedfile):
-                #     import pdb; pdb.set_trace()
-                if md5sum(targetfile) != md5sum(expectedfile):
+                md5expected = md5sum(expectedfile, ignore_newlines=ON_WINDOWS)
+                md5target = md5sum(targetfile, ignore_newlines=ON_WINDOWS)
+                if md5target != md5expected:
+                    # import pdb; pdb.set_trace()
                     with open(targetfile) as target:
                         content = target.read()
                     assert False, 'wrong result produced for file "{}":\n{}'.format(
